@@ -24,6 +24,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: asner.h,v $
+ * Revision 1.38  2003/04/22 23:39:09  craigs
+ * Changed some functions from protected to public for MacOSX. Thanks to Hugo Santos
+ *
+ * Revision 1.37  2003/04/17 14:44:44  craigs
+ * Removed MacOS specific defines to make some attributes public
+ * Thanks to Hugo Santos and apologies to Roger Hardiman
+ *
  * Revision 1.36  2003/02/26 01:57:44  robertj
  * Added XML encoding rules to ASN system, thanks Federico Pinna
  *
@@ -834,18 +841,14 @@ class PASN_Choice : public PASN_Object
     void EncodeXER(PXER_Stream &) const;
 #endif
 
-#ifdef P_MACOSX
-  #warning making PASN_Choice public
-  public:
-#else
+    PASN_Choice & operator=(const PASN_Choice & other);
+
   protected:
-#endif
     PASN_Choice(unsigned nChoices = 0, BOOL extend = FALSE);
     PASN_Choice(unsigned tag, TagClass tagClass, unsigned nChoices, BOOL extend);
     PASN_Choice(unsigned tag, TagClass tagClass, unsigned nChoices, BOOL extend, const PString & nameSpec);
 
     PASN_Choice(const PASN_Choice & other);
-    PASN_Choice & operator=(const PASN_Choice & other);
 
     BOOL CheckCreate() const;
 
@@ -972,17 +975,13 @@ class PASN_Array : public PASN_ConstrainedObject
 
     virtual PASN_Object * CreateObject() const = 0;
 
-#ifdef P_MACOSX
-  #warning making PASN_Array public
-  public:
-#else
+    PASN_Array & operator=(const PASN_Array & other);
+
   protected:
-#endif
     PASN_Array(unsigned tag = UniversalSequence,
                TagClass tagClass = UniversalTagClass);
 
     PASN_Array(const PASN_Array & other);
-    PASN_Array & operator=(const PASN_Array & other);
 
     PASN_ObjectArray array;
 };
