@@ -8,6 +8,11 @@
  * Copyright 2002 Equivalence
  *
  * $Log: main.cxx,v $
+ * Revision 1.6  2003/04/15 03:00:41  robertj
+ * Added array support to XML/RPC
+ * Fixed XML/RPC parsing when lots of white space in raw XML, caused by
+ *   big fix to base XML parser not returning internal data elements.
+ *
  * Revision 1.5  2002/12/04 02:09:17  robertj
  * Changed macro name prefix to PXMLRPC
  *
@@ -35,10 +40,15 @@
 
     http://www.mirrorproject.com/xmlrpc mirror.Random
 
+    http://xmlrpc.usefulinc.com/demo/server.php system.listMethods
     http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoString "A test!"
     -i http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoInteger 12
     -f http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoFloat 3.121
+    -a http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoStringArray One Two Three Four
+    -a -i http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoIntegerArray 11 222 3333 44444
+    -a -f http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoIntegerArray 1.1 22.23 333.333 4444.4444
     -s http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoStruct first 1st second 2nd third 3rd
+    -a -s http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoStructArray first 1st second 2nd third 3rd "," fourth 4th fifth 5th "," sixth 6th "," seventh 7th eigth 8th ninth 9th tenth 10th
     --echo-struct http://xmlrpc.usefulinc.com/demo/server.php interopEchoTests.echoStruct
     -s http://10.0.2.13:6666/RPC2 Function1 key value
 
@@ -56,14 +66,18 @@ PXMLRPC_STRUCT_BEGIN(NestedStruct)
     PXMLRPC_INTEGER (NestedStruct, int, another_integer);
 PXMLRPC_STRUCT_END()
 
-PXMLRPC_STRUCT_BEGIN(TestStruct)
-    PXMLRPC_STRING  (TestStruct, PString, a_string);
-    PXMLRPC_INTEGER (TestStruct, int, an_integer);
-    PXMLRPC_BOOLEAN (TestStruct, BOOL, a_boolean);
-    PXMLRPC_DOUBLE  (TestStruct, double, a_float);
-    PXMLRPC_DATETIME(TestStruct, PTime, a_date);
-    PXMLRPC_BINARY  (TestStruct, PBYTEArray, a_binary);
-    PXMLRPC_STRUCT  (TestStruct, NestedStruct, nested_struct);
+PXMLRPC_STRUCT_BEGIN     (TestStruct)
+    PXMLRPC_STRING_INIT  (TestStruct, PString, a_string, "A string!");
+    PXMLRPC_INTEGER_INIT (TestStruct, int, an_integer, 12);
+    PXMLRPC_BOOLEAN_INIT (TestStruct, BOOL, a_boolean, TRUE);
+    PXMLRPC_DOUBLE_INIT  (TestStruct, double, a_float, 3.14159);
+    PXMLRPC_DATETIME     (TestStruct, PTime, a_date);
+    PXMLRPC_BINARY       (TestStruct, PBYTEArray, a_binary);
+    PXMLRPC_ARRAY_STRING (TestStruct, PStringArray, PCaselessString, a_string_array);
+    PXMLRPC_ARRAY_INTEGER(TestStruct, int, an_integer_array);
+    PXMLRPC_ARRAY_DOUBLE (TestStruct, float, a_float_array);
+    PXMLRPC_STRUCT       (TestStruct, NestedStruct, nested_struct);
+    PXMLRPC_ARRAY_STRUCT (TestStruct, NestedStruct, array_struct);
 PXMLRPC_STRUCT_END()
  
 
@@ -82,7 +96,8 @@ void XMLRPCApp::Main()
   PINDEX i;
   PArgList & args = GetArguments();
 
-  args.Parse("d-debug."
+  args.Parse("a-array."
+             "d-debug."
              "f-float."
              "i-integer."
              "o-output:"
@@ -109,20 +124,70 @@ void XMLRPCApp::Main()
 
   if (args.HasOption("echo-struct")) {
     TestStruct ts;
-    ts.a_string = "A string!";
-    ts.an_integer = 12;
-    ts.a_float = 3.14159;
-    ts.a_boolean = TRUE;
-    ts.a_date = PTime() - PTimeInterval(0, 0, 0, 0, 5);
+    ts.a_date -= PTimeInterval(0, 0, 0, 0, 5);
+
     ts.a_binary.SetSize(10);
     for (i = 0; i < 10; i++)
       ts.a_binary[i] = (BYTE)(i+1);
+
+    ts.a_string_array.SetSize(3);
+    ts.a_string_array[0] = "first";
+    ts.a_string_array[1] = "second";
+    ts.a_string_array[2] = "third";
+
+    ts.an_integer_array.SetSize(7);
+    for (i = 0; i < ts.an_integer_array.GetSize(); i++)
+      ts.an_integer_array[i] = i+1;
+
+    ts.a_float_array.SetSize(5);
+    for (i = 0; i < ts.a_float_array.GetSize(); i++)
+      ts.a_float_array[i] = (float)(1.0/(i+2));
+
     ts.nested_struct.another_string = "Another string!";
     ts.nested_struct.another_integer = 345;
+
+    ts.array_struct.SetSize(2);
+    ts.array_struct.SetAt(0, new NestedStruct);
+    ts.array_struct[0].another_string = "Structure one";
+    ts.array_struct[0].another_integer = 11111;
+    ts.array_struct.SetAt(1, new NestedStruct);
+    ts.array_struct[1].another_string = "Structure two";
+    ts.array_struct[1].another_integer = 22222;
     request.AddParam(ts);
   }
   else {
-    if (args.HasOption('s')) {
+    if (args.HasOption('a')) {
+      if (args.HasOption('s')) {
+        PArray<PStringToString> array;
+        PStringToString dict;
+        PString key;
+        for (i = 2; i < args.GetCount(); i++) {
+          if (args[i] == ",") {
+            array.SetAt(array.GetSize(), new PStringToString(dict));
+            dict = PStringToString();
+            key = PString::Empty();
+          }
+          else if (key.IsEmpty())
+            key = args[i];
+          else {
+            dict.SetAt(key, args[i]);
+            key = PString::Empty();
+          }
+        }
+
+        if (!dict.IsEmpty())
+          array.SetAt(array.GetSize(), new PStringToString(dict));
+
+        request.AddArray(array);
+      }
+      else if (args.HasOption('i'))
+        request.AddArray(args.GetParameters(2), "int");
+      else if (args.HasOption('f'))
+        request.AddArray(args.GetParameters(2), "double");
+      else
+        request.AddArray(args.GetParameters(2));
+    }
+    else if (args.HasOption('s')) {
       PStringToString dict;
       for (i = 2; (i+1) < args.GetCount(); i += 2) {
         PString key   = args[i];
@@ -164,13 +229,18 @@ void XMLRPCApp::Main()
     PString val;
     if (response.GetParam(i, type, val)) {
       cout << type << " = ";
-      if (type != "struct")
-        cout << val;
-      else {
+      if (type == "struct") {
         PStringToString dict;
         response.GetParam(i, dict);
         cout << '\n' << dict;
       }
+      else if (type == "array") {
+        PStringArray array;
+        response.GetParam(i, array);
+        cout << '\n' << setfill('\n') << array << setfill(' ');
+      }
+      else
+        cout << val;
     }
     else
       cout << "error: " << response.GetFaultText();
