@@ -24,6 +24,42 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: http.cxx,v $
+ * Revision 1.90  2003/06/05 00:15:54  rjongbloed
+ * Fixed callto bug created by previous patch.
+ *
+ * Revision 1.89  2003/06/04 01:42:05  rjongbloed
+ * Fixed h323 scheme, does not have a "password" field.
+ *
+ * Revision 1.88  2003/06/02 02:46:45  rjongbloed
+ * Fixed issue with callto URL parsing incorrect username field.
+ * Added automatic removal of illegal (though common) "//" in callto URL.
+ *
+ * Revision 1.87  2003/05/05 07:30:17  craigs
+ * Fixed problem with URLs that do not specify schemes
+ *
+ * Revision 1.86  2003/05/02 13:50:23  craigs
+ * Fixed a problem with callto:localhost
+ *
+ * Revision 1.85  2003/05/02 13:20:33  craigs
+ * Fixed callto problems
+ *
+ * Revision 1.84  2003/04/28 04:41:22  robertj
+ * Changed URL parsing so if a default scheme is present then explicit scheme
+ *   must be "known" to avoid ambiguity with host:port parsing.
+ *
+ * Revision 1.83  2003/04/10 00:13:56  robertj
+ * Fixed correct decoding of user/password/host/port field, for non h323 schemes.
+ *
+ * Revision 1.82  2003/04/08 06:28:14  craigs
+ * Fixed introduced problem with HTTP server mistaking relative URLs for proxy requests
+ *
+ * Revision 1.81  2003/04/04 08:03:55  robertj
+ * Fixed special case of h323 URL default port changing depending on
+ *   if it the host is an endpoint or gatekeeper.
+ *
+ * Revision 1.80  2003/04/04 05:18:08  robertj
+ * Added "callto", "tel" and fixed "h323" URL types.
+ *
  * Revision 1.79  2002/12/02 00:17:03  robertj
  * Fixed URL parsing/display problems with non-path URL type eg mailto
  *
@@ -327,13 +363,17 @@
 #define DEFAULT_RTSPU_PORT	554
 #define DEFAULT_PROSPERO_PORT	1525
 #define	DEFAULT_H323_PORT       1720
+#define	DEFAULT_H323RAS_PORT    1719
 #define	DEFAULT_SIP_PORT        5060
 
 
 struct schemeStruct {
   const char * name;
-  BOOL hasUserPassword;
+  BOOL hasUsername;
+  BOOL hasPassword;
   BOOL hasHostPort;
+  BOOL defaultToUserIfNoAt;
+  BOOL defaultHostToLocal;
   BOOL hasQuery;
   BOOL hasParameters;
   BOOL hasFragments;
@@ -345,62 +385,64 @@ struct schemeStruct {
 #define DEFAULT_SCHEME 0
 #define FILE_SCHEME    1
 
-static schemeStruct const schemeInfo[] = {
-//  scheme       user   host   query  params frags  path   rel    port
-  { "http",      TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTP_PORT     }, // Must be first
-  { "file",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, 0                     }, // Must be second
-  { "https",     FALSE, TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTPS_PORT    },
-  { "gopher",    FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_GOPHER_PORT   },
-  { "wais",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_WAIS_PORT     },
-  { "nntp",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_NNTP_PORT     },
-  { "prospero",  FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_PROSPERO_PORT },
-  { "rtsp",      FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSP_PORT     },
-  { "rtspu",     FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSPU_PORT    },
-
-  { "ftp",       TRUE,  TRUE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_FTP_PORT      },
-  { "telnet",    TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, DEFAULT_TELNET_PORT   },
-  { "mailto",    FALSE, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, 0                     },
-  { "news",      FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0                     },
-  { "h323",      TRUE,  TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_H323_PORT     },
-  { "sip",       TRUE,  TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_SIP_PORT      },
-  { NULL,        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0                     }
+static schemeStruct const SchemeTable[] = {
+//  scheme       user   pass   host   @def   defhost query  params frags  path   rel    port
+  { "http",      TRUE,  TRUE,  TRUE,  FALSE, TRUE,   TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTP_PORT     }, // Must be first
+  { "file",      FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, 0                     }, // Must be second
+  { "https",     FALSE, FALSE, TRUE,  FALSE, TRUE,   TRUE,  TRUE,  TRUE,  TRUE,  TRUE,  DEFAULT_HTTPS_PORT    },
+  { "gopher",    FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_GOPHER_PORT   },
+  { "wais",      FALSE, FALSE, TRUE,  FALSE, FALSE,  FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_WAIS_PORT     },
+  { "nntp",      FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_NNTP_PORT     },
+  { "prospero",  FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_PROSPERO_PORT },
+  { "rtsp",      FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSP_PORT     },
+  { "rtspu",     FALSE, FALSE, TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_RTSPU_PORT    },
+  { "ftp",       TRUE,  TRUE,  TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, TRUE,  FALSE, DEFAULT_FTP_PORT      },
+  { "telnet",    TRUE,  TRUE,  TRUE,  FALSE, TRUE,   FALSE, FALSE, FALSE, FALSE, FALSE, DEFAULT_TELNET_PORT   },
+  { "mailto",    FALSE, FALSE, FALSE, FALSE, TRUE,   TRUE,  FALSE, FALSE, FALSE, FALSE, 0                     },
+  { "news",      FALSE, FALSE, FALSE, FALSE, TRUE,   FALSE, FALSE, FALSE, FALSE, FALSE, 0                     },
+  { "h323",      TRUE,  FALSE, TRUE,  TRUE,  FALSE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_H323_PORT     },
+  { "sip",       TRUE,  TRUE,  TRUE,  FALSE, FALSE,  FALSE, TRUE,  FALSE, FALSE, FALSE, DEFAULT_SIP_PORT      },
+  { "tel",       FALSE, FALSE, FALSE, TRUE,  FALSE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 0                     },
+  { "fax",       FALSE, FALSE, FALSE, TRUE,  FALSE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 0                     },
+  { "callto",    FALSE, FALSE, FALSE, TRUE,  FALSE,  FALSE, TRUE,  FALSE, FALSE, FALSE, 0                     },
+  { NULL,        FALSE, FALSE, FALSE, FALSE, FALSE,  FALSE, FALSE, FALSE, FALSE, FALSE, 0                     }
 };
 
-static const schemeStruct & GetSchemeInfo(const PCaselessString & scheme)
+static const schemeStruct * GetSchemeInfo(const PCaselessString & scheme)
 {
   PINDEX i;
-  for (i = 0; schemeInfo[i].name != NULL; i++) {
-    if (scheme == schemeInfo[i].name)
-      return schemeInfo[i];
+  for (i = 0; SchemeTable[i].name != NULL; i++) {
+    if (scheme == SchemeTable[i].name)
+      return &SchemeTable[i];
   }
-  return schemeInfo[i];
+  return NULL;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 // PURL
 
 PURL::PURL()
-  : scheme(schemeInfo[DEFAULT_SCHEME].name),
+  : scheme(SchemeTable[DEFAULT_SCHEME].name),
     port(0),
     relativePath(FALSE)
 {
 }
 
 
-PURL::PURL(const char * str)
+PURL::PURL(const char * str, const char * defaultScheme)
 {
-  Parse(str);
+  Parse(str, defaultScheme);
 }
 
 
-PURL::PURL(const PString & str)
+PURL::PURL(const PString & str, const char * defaultScheme)
 {
-  Parse(str);
+  Parse(str, defaultScheme);
 }
 
 
 PURL::PURL(const PFilePath & filePath)
-  : scheme(schemeInfo[FILE_SCHEME].name),
+  : scheme(SchemeTable[FILE_SCHEME].name),
     port(0),
     relativePath(FALSE)
 {
@@ -537,7 +579,7 @@ void PURL::SplitQueryVars(const PString & queryStr, PStringToString & queryVars)
 }
 
 
-void PURL::Parse(const char * cstr)
+void PURL::Parse(const char * cstr, const char * defaultScheme)
 {
   hostname = PCaselessString();
   pathStr = username = password = fragment = PString::Empty();
@@ -553,32 +595,103 @@ void PURL::Parse(const char * cstr)
 
   PINDEX pos;
 
-  static const PString reservedChars = "=;/#?";
-
-  // determine if the URL has a scheme
-  scheme = PString::Empty();
-  if (isalpha(url[0])) {
-    for (pos = 0; url[pos] != '\0' &&
-                          reservedChars.Find(url[pos]) == P_MAX_INDEX; pos++) {
-      if (url[pos] == ':') {
-        scheme = url.Left(pos);
-        url.Delete(0, pos+1);
-        break;
-      }
-    }
-  }
-
-  // if there is no scheme, then default to http for the local
-  // on the default port
-  if (scheme.IsEmpty())
-    scheme = schemeInfo[DEFAULT_SCHEME].name;
-
   // get information which tells us how to parse URL for this
   // particular scheme
-  const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
+  const schemeStruct * schemeInfo = NULL;
     
+  // determine if the URL has a scheme
+  scheme = PString::Empty();
+
+  // Character set as per RFC2396
+  pos = 0;
+  while (isalnum(url[pos]) || url[pos] == '+' || url[pos] == '-' || url[pos] == '.')
+    pos++;
+
+  // Have explicit scheme
+  if (url[pos] == ':') {
+    schemeInfo = GetSchemeInfo(url.Left(pos));
+    if (schemeInfo == NULL && defaultScheme == NULL)
+      schemeInfo = &SchemeTable[PARRAYSIZE(SchemeTable)-1];
+    if (schemeInfo != NULL)
+      url.Delete(0, pos+1);
+
+    // if there is no scheme, then use default
+    if (schemeInfo == NULL && defaultScheme != NULL)
+      schemeInfo = GetSchemeInfo(defaultScheme);
+    if (schemeInfo == NULL)
+      schemeInfo = &SchemeTable[DEFAULT_SCHEME];
+    scheme = schemeInfo->name;
+  }
+
+  // if we could not match a scheme, then use the default scheme
+  if (schemeInfo == NULL) 
+    schemeInfo = &SchemeTable[DEFAULT_SCHEME];
+
+
+  // Super special case!
+  if (scheme *= "callto") {
+    // Actually not part of MS spec, but a lot of people put in the // into
+    // the URL, so we take it out of it is there.
+    if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/')
+      url.Delete(0, 2);
+
+    // For some bizarre reason callto uses + instead of ; for paramters
+    // We do a loop so that phone numbers of the form +61243654666 still work
+    do {
+      pos = url.Find('+');
+    } while (pos != P_MAX_INDEX && isdigit(url[pos+1]));
+
+    if (pos != P_MAX_INDEX) {
+      SplitVars(url(pos+1, P_MAX_INDEX), paramVars, '+', '=');
+      url.Delete(pos, P_MAX_INDEX);
+    }
+
+    hostname = paramVars("gateway");
+    if (!hostname)
+      username = UntranslateString(url, LoginTranslation);
+    else {
+      PCaselessString type = paramVars("type");
+      if (type == "directory") {
+        pos = url.Find('/');
+        if (pos == P_MAX_INDEX)
+          username = UntranslateString(url, LoginTranslation);
+        else {
+          hostname = UntranslateString(url.Left(pos), LoginTranslation);
+          username = UntranslateString(url.Mid(pos+1), LoginTranslation);
+        }
+      }
+      else {
+        // Now look for an @ and split user and host
+        pos = url.Find('@');
+        if (pos != P_MAX_INDEX) {
+          username = UntranslateString(url.Left(pos), LoginTranslation);
+          hostname = UntranslateString(url.Mid(pos+1), LoginTranslation);
+        }
+        else {
+          if (type == "ip" || type == "host")
+            hostname = UntranslateString(url, LoginTranslation);
+          else
+            username = UntranslateString(url, LoginTranslation);
+        }
+      }
+    }
+
+    // Allow for [ipv6] form
+    pos = hostname.Find(']');
+    if (pos == P_MAX_INDEX)
+      pos = 0;
+    pos = hostname.Find(':', pos);
+    if (pos != P_MAX_INDEX) {
+      port = (WORD)hostname.Mid(pos+1).AsUnsigned();
+      hostname.Delete(pos, P_MAX_INDEX);
+    }
+
+    password = paramVars("password");
+    return;
+  }
+
   // if the URL should have leading slash, then remove it if it has one
-  if (schemeInfo.hasHostPort && schemeInfo.hasPath) {
+  if (schemeInfo != NULL && schemeInfo->hasHostPort && schemeInfo->hasPath) {
     if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/')
       url.Delete(0, 2);
     else
@@ -586,92 +699,132 @@ void PURL::Parse(const char * cstr)
   }
 
   // parse user/password/host/port
-  if (!relativePath && schemeInfo.hasHostPort) {
-    pos = url.FindOneOf("/;?#");
+  if (!relativePath && schemeInfo->hasHostPort) {
+    PString endHostChars;
+    if (schemeInfo->hasPath)
+      endHostChars += '/';
+    if (schemeInfo->hasQuery)
+      endHostChars += '?';
+    if (schemeInfo->hasParameters)
+      endHostChars += ';';
+    if (schemeInfo->hasFragments)
+      endHostChars += '#';
+    if (endHostChars.IsEmpty())
+      pos = P_MAX_INDEX;
+    else
+      pos = url.FindOneOf(endHostChars);
+
     PString uphp = url.Left(pos);
     if (pos != P_MAX_INDEX)
       url.Delete(0, pos);
     else
-      url = PString();
-
-    // if the URL is of type HostOnly, then this is the hostname
-    if (schemeInfo.defaultPort == 0)
-      hostname = UntranslateString(uphp, LoginTranslation);
+      url = PString::Empty();
 
     // if the URL is of type UserPasswordHostPort, then parse it
-    if (schemeInfo.hasUserPassword) {
+    if (schemeInfo->hasUsername) {
       // extract username and password
       PINDEX pos2 = uphp.Find('@');
-      if (pos2 != P_MAX_INDEX && pos2 > 0) {
-        PINDEX pos3 = uphp.Find(':');
-        // if no password...
-        if (pos3 > pos2)
-          username = UntranslateString(uphp(0, pos2-1), LoginTranslation);
-        else {
-          username = UntranslateString(uphp(0, pos3-1), LoginTranslation);
-          password = UntranslateString(uphp(pos3+1, pos2-1), LoginTranslation);
-        }
-        uphp.Delete(0, pos2+1);
+      PINDEX pos3 = P_MAX_INDEX;
+      if (schemeInfo->hasPassword)
+        pos3 = uphp.Find(':');
+      switch (pos2) {
+        case 0 :
+          uphp.Delete(0, 1);
+          break;
+
+        case P_MAX_INDEX :
+          if (schemeInfo->defaultToUserIfNoAt) {
+            if (pos3 == P_MAX_INDEX)
+              username = UntranslateString(uphp, LoginTranslation);
+            else {
+              username = UntranslateString(uphp.Left(pos3), LoginTranslation);
+              password = UntranslateString(uphp.Mid(pos3+1), LoginTranslation);
+            }
+            uphp = PString::Empty();
+          }
+          break;
+
+        default :
+          if (pos3 > pos2)
+            username = UntranslateString(uphp.Left(pos2), LoginTranslation);
+          else {
+            username = UntranslateString(uphp.Left(pos3), LoginTranslation);
+            password = UntranslateString(uphp(pos3+1, pos2-1), LoginTranslation);
+          }
+          uphp.Delete(0, pos2+1);
       }
     }
 
-    // determine if the URL has a port number
-    if (schemeInfo.defaultPort != 0) {
-      pos = uphp.Find(':');
-      if (pos == P_MAX_INDEX) {
+    // if the URL does not have a port, then this is the hostname
+    if (schemeInfo->defaultPort == 0)
+      hostname = UntranslateString(uphp, LoginTranslation);
+    else {
+      // determine if the URL has a port number
+      // Allow for [ipv6] form
+      pos = uphp.Find(']');
+      if (pos == P_MAX_INDEX)
+        pos = 0;
+      pos = uphp.Find(':', pos);
+      if (pos == P_MAX_INDEX)
         hostname = UntranslateString(uphp, LoginTranslation);
-        port = schemeInfo.defaultPort;
-      }
       else {
         hostname = UntranslateString(uphp.Left(pos), LoginTranslation);
-        port = (WORD)uphp(pos+1, P_MAX_INDEX).AsInteger();
+        port = (WORD)uphp.Mid(pos+1).AsUnsigned();
       }
 
-      if (hostname.IsEmpty())
+      if (hostname.IsEmpty() && schemeInfo->defaultHostToLocal)
         hostname = PIPSocket::GetHostName();
     }
   }
 
-  if (schemeInfo.hasQuery) {
+  if (schemeInfo->hasQuery) {
     // chop off any trailing query
     pos = url.Find('?');
-    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+    if (pos != P_MAX_INDEX) {
       SplitQueryVars(url(pos+1, P_MAX_INDEX), queryVars);
       url.Delete(pos, P_MAX_INDEX);
     }
   }
 
-  if (schemeInfo.hasParameters) {
+  if (schemeInfo->hasParameters) {
     // chop off any trailing parameters
     pos = url.Find(';');
-    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+    if (pos != P_MAX_INDEX) {
       SplitVars(url(pos+1, P_MAX_INDEX), paramVars, ';', '=');
       url.Delete(pos, P_MAX_INDEX);
     }
   }
 
-  if (schemeInfo.hasFragments) {
+  if (schemeInfo->hasFragments) {
     // chop off any trailing fragment
     pos = url.Find('#');
-    if (pos != P_MAX_INDEX /* && pos > 0 */) {
+    if (pos != P_MAX_INDEX) {
       fragment = UntranslateString(url(pos+1, P_MAX_INDEX), PathTranslation);
       url.Delete(pos, P_MAX_INDEX);
     }
   }
 
-  if (schemeInfo.hasPath)
+  if (schemeInfo->hasPath)
     SetPathStr(url);   // the hierarchy is what is left
   else {
   // if the rest of the URL isn't a path, then we are finished!
     pathStr = UntranslateString(url, PathTranslation);
     Recalculate();
   }
+
+  if (port == 0 && schemeInfo->defaultPort != 0 && !relativePath) {
+    // Yes another horrible, horrible special case!
+    if (scheme == "h323" && paramVars("type") == "gk")
+      port = DEFAULT_H323RAS_PORT;
+    else
+      port = schemeInfo->defaultPort;
+  }
 }
 
 
 PFilePath PURL::AsFilePath() const
 {
-  if (scheme != schemeInfo[FILE_SCHEME].name)
+  if (scheme != SchemeTable[FILE_SCHEME].name)
     return PString::Empty();
 
   PStringStream str;
@@ -705,34 +858,36 @@ PString PURL::AsString(UrlFormat fmt) const
   if (scheme.IsEmpty())
     return PString::Empty();
 
-  const schemeStruct & schemeInfo = GetSchemeInfo(scheme);
+  const schemeStruct * schemeInfo = GetSchemeInfo(scheme);
+  if (schemeInfo == NULL)
+    schemeInfo = &SchemeTable[PARRAYSIZE(SchemeTable)-1];
 
   if (fmt == HostPortOnly) {
     str << scheme << ':';
 
     if (relativePath) {
-      if (schemeInfo.relativeImpliesScheme)
+      if (schemeInfo->relativeImpliesScheme)
         return PString::Empty();
       return str;
     }
 
-    if (schemeInfo.hasPath && schemeInfo.hasHostPort)
+    if (schemeInfo->hasPath && schemeInfo->hasHostPort)
       str << "//";
 
-    if (schemeInfo.hasUserPassword) {
+    if (schemeInfo->hasUsername) {
       if (!username) {
         str << TranslateString(username, LoginTranslation);
-        if (!password)
+        if (schemeInfo->hasPassword && !password)
           str << ':' << TranslateString(password, LoginTranslation);
         str << '@';
       }
     }
 
-    if (schemeInfo.hasHostPort)
+    if (schemeInfo->hasHostPort)
       str << hostname;
 
-    if (schemeInfo.defaultPort != 0) {
-      if (port != schemeInfo.defaultPort)
+    if (schemeInfo->defaultPort != 0) {
+      if (port != schemeInfo->defaultPort)
         str << ':' << port;
     }
 
@@ -740,7 +895,7 @@ PString PURL::AsString(UrlFormat fmt) const
   }
 
   // URIOnly and PathOnly
-  if (schemeInfo.hasPath) {
+  if (schemeInfo->hasPath) {
     for (i = 0; i < path.GetSize(); i++) {
       if (i > 0 || !relativePath)
         str << '/';
@@ -933,7 +1088,7 @@ BOOL PURL::OpenBrowser(const PString & url)
 void PURL::Recalculate()
 {
   if (scheme.IsEmpty())
-    scheme = schemeInfo[0].name;
+    scheme = SchemeTable[DEFAULT_SCHEME].name;
 
   urlString = AsString(HostPortOnly) + AsString(URIOnly);
 }
