@@ -27,6 +27,22 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: contain.cxx,v $
+ * Revision 1.133  2003/05/14 00:48:33  rjongbloed
+ * Added constructor to string lists/arrays etc that takes a single PString.
+ * Fixed bug in doing a MakeUnique on a container, it would lose the
+ *   DisallowDeleteObjects flag.
+ *
+ * Revision 1.132  2003/04/28 09:14:14  robertj
+ * Fixed bad sign extension problem in PBYTEArray output
+ *
+ * Revision 1.131  2003/04/15 07:08:37  robertj
+ * Changed read and write from streams for base array classes so operates in
+ *   the same way for both PIntArray and PArray<int> etc
+ *
+ * Revision 1.130  2003/03/31 01:24:23  robertj
+ * Added ReadFrom functions for standard container classes such as
+ *   PIntArray and PStringList etc
+ *
  * Revision 1.129  2003/03/05 08:48:32  robertj
  * Added PStringArray::ToCharAray() function at suggestion of Ravelli Rossano
  *
@@ -583,7 +599,7 @@ BOOL PContainer::MakeUnique()
     return TRUE;
 
   reference->count--;
-  reference = new Reference(GetSize());
+  reference = new Reference(*reference);
   return FALSE;
 }
 
@@ -663,6 +679,33 @@ void PAbstractArray::CloneContents(const PAbstractArray * array)
 }
 
 
+void PAbstractArray::PrintOn(ostream & strm) const
+{
+  char separator = strm.fill();
+  int width = strm.width();
+  for (PINDEX  i = 0; i < GetSize(); i++) {
+    if (i > 0 && separator != '\0')
+      strm << separator;
+    strm.width(width);
+    PrintElementOn(strm, i);
+  }
+  if (separator == '\n')
+    strm << '\n';
+}
+
+
+void PAbstractArray::ReadFrom(istream & strm)
+{
+  PINDEX i = 0;
+  while (strm.good()) {
+    ReadElementFrom(strm, i);
+    if (!strm.fail())
+      i++;
+  }
+  SetSize(i);
+}
+
+
 PObject::Comparison PAbstractArray::Compare(const PObject & obj) const
 {
   PAssert(obj.IsDescendant(PAbstractArray::Class()), PInvalidCast);
@@ -795,90 +838,13 @@ BOOL PAbstractArray::Concatenate(const PAbstractArray & array)
 }
 
 
-void PAbstractArray::PrintNumbersOn(ostream & strm, PINDEX size, BOOL is_signed) const
+void PAbstractArray::PrintElementOn(ostream & /*stream*/, PINDEX /*index*/) const
 {
-  PINDEX line_width = strm.width();
-  if (line_width == 0)
-    line_width = 16/size;
-  strm.width(0);
-
-  PINDEX indent = strm.precision();
-
-  PINDEX val_width;
-  switch (strm.flags()&ios::basefield) {
-    case ios::hex :
-      val_width = size*2;
-      is_signed = FALSE;
-      break;
-    case ios::oct :
-      val_width = ((size*8)+2)/3;
-      is_signed = FALSE;
-      break;
-    default :
-      switch (size) {
-        case 1 :
-          val_width = 3;
-          break;
-        case 2 :
-          val_width = 5;
-          break;
-        default :
-          val_width = 10;
-          break;
-      }
-      if (is_signed)
-        val_width++;
-  }
-
-  long mask = -1;
-  if (size < (PINDEX)sizeof(mask))
-    mask = (1L << (size*8)) - 1;
-
-  PINDEX i = 0;
-  while (i < GetSize()) {
-    if (i > 0)
-      strm << '\n';
-    PINDEX j;
-    for (j = 0; j < indent; j++)
-      strm << ' ';
-    for (j = 0; j < line_width; j++) {
-      if (j == line_width/2)
-        strm << ' ';
-      if (i+j < GetSize()) {
-        strm << setw(val_width);
-        if (is_signed)
-          strm << GetNumberValueAt(i+j);
-        else
-          strm << (DWORD)(GetNumberValueAt(i+j)&mask);
-      }
-      else {
-        PINDEX k;
-        for (k = 0; k < val_width; k++)
-          strm << ' ';
-      }
-      strm << ' ';
-    }
-    if ((strm.flags()&ios::floatfield) != ios::fixed) {
-      strm << "  ";
-      for (j = 0; j < line_width; j++) {
-        if (i+j < GetSize()) {
-          long val = GetNumberValueAt(i+j);
-          if (val >= 0 && val < 256 && isprint(val))
-            strm << (char)val;
-          else
-            strm << '.';
-        }
-      }
-    }
-    i += line_width;
-  }
 }
 
 
-long PAbstractArray::GetNumberValueAt(PINDEX) const
+void PAbstractArray::ReadElementFrom(istream & /*stream*/, PINDEX /*index*/)
 {
-  PAssertAlways(PUnimplementedFunction);
-  return 0;
 }
 
 
@@ -904,87 +870,85 @@ void PCharArray::PrintOn(ostream & strm) const
 }
 
 
-void PShortArray::PrintOn(ostream & strm) const
+void PCharArray::ReadFrom(istream &strm)
 {
-  PrintNumbersOn(strm, sizeof(short), TRUE);
-}
+  PINDEX size = 0;
+  SetSize(size+100);
 
+  while (strm.good()) {
+    strm >> theArray[size++];
+    if (size >= GetSize())
+      SetSize(size+100);
+  }
 
-long PShortArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((short *)theArray)[idx];
-}
-
-
-void PIntArray::PrintOn(ostream & strm) const
-{
-  PrintNumbersOn(strm, sizeof(int), TRUE);
-}
-
-
-long PIntArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((int *)theArray)[idx];
-}
-
-
-void PLongArray::PrintOn(ostream & strm) const
-{
-  PrintNumbersOn(strm, sizeof(long), TRUE);
-}
-
-
-long PLongArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((long *)theArray)[idx];
+  SetSize(size);
 }
 
 
 void PBYTEArray::PrintOn(ostream & strm) const
 {
-  PrintNumbersOn(strm, sizeof(BYTE), FALSE);
+  PINDEX line_width = strm.width();
+  if (line_width == 0)
+    line_width = 16;
+  strm.width(0);
+
+  PINDEX indent = strm.precision();
+
+  PINDEX val_width = ((strm.flags()&ios::basefield) == ios::hex) ? 2 : 3;
+
+  PINDEX i = 0;
+  while (i < GetSize()) {
+    if (i > 0)
+      strm << '\n';
+    PINDEX j;
+    for (j = 0; j < indent; j++)
+      strm << ' ';
+    for (j = 0; j < line_width; j++) {
+      if (j == line_width/2)
+        strm << ' ';
+      if (i+j < GetSize())
+        strm << setw(val_width) << (theArray[i+j]&0xff);
+      else {
+        PINDEX k;
+        for (k = 0; k < val_width; k++)
+          strm << ' ';
+      }
+      strm << ' ';
+    }
+    if ((strm.flags()&ios::floatfield) != ios::fixed) {
+      strm << "  ";
+      for (j = 0; j < line_width; j++) {
+        if (i+j < GetSize()) {
+          unsigned val = theArray[i+j]&0xff;
+          if (isprint(val))
+            strm << (char)val;
+          else
+            strm << '.';
+        }
+      }
+    }
+    i += line_width;
+  }
 }
 
 
-long PBYTEArray::GetNumberValueAt(PINDEX idx) const
+void PBYTEArray::ReadFrom(istream &strm)
 {
-  return ((BYTE *)theArray)[idx];
-}
+  PINDEX size = 0;
+  SetSize(size+100);
 
+  while (strm.good()) {
+    unsigned v;
+    strm >> v;
+    theArray[size] = (BYTE)v;
+    if (!strm.fail()) {
+      size++;
+      if (size >= GetSize())
+        SetSize(size+100);
+    }
+  }
 
-void PWORDArray::PrintOn(ostream & strm) const
-{
-  PrintNumbersOn(strm, sizeof(WORD), FALSE);
-}
-
-
-long PWORDArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((WORD *)theArray)[idx];
-}
-
-
-void PUnsignedArray::PrintOn(ostream & strm) const
-{
-  PrintNumbersOn(strm, sizeof(unsigned), FALSE);
-}
-
-
-long PUnsignedArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((unsigned *)theArray)[idx];
-}
-
-
-void PDWORDArray::PrintOn(ostream & strm) const
-{
-  PrintNumbersOn(strm, sizeof(DWORD), FALSE);
-}
-
-
-long PDWORDArray::GetNumberValueAt(PINDEX idx) const
-{
-  return ((DWORD *)theArray)[idx];
+  SetSize(size);
 }
 
 
@@ -2548,6 +2512,13 @@ PStringArray::PStringArray(PINDEX count, char const * const * strarr, BOOL casel
 }
 
 
+PStringArray::PStringArray(const PString & str)
+{
+  SetSize(1);
+  (*theArray)[0] = new PString(str);
+}
+
+
 PStringArray::PStringArray(const PStringList & list)
 {
   SetSize(list.GetSize());
@@ -2564,6 +2535,16 @@ PStringArray::PStringArray(const PSortedStringList & list)
 }
 
 
+void PStringArray::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    AppendString(str);
+  }
+}
+
+
 PString & PStringArray::operator[](PINDEX index)
 {
   PASSERTINDEX(index);
@@ -2627,6 +2608,12 @@ PStringList::PStringList(PINDEX count, char const * const * strarr, BOOL caseles
 }
 
 
+PStringList::PStringList(const PString & str)
+{
+  AppendString(str);
+}
+
+
 PStringList::PStringList(const PStringArray & array)
 {
   for (PINDEX i = 0; i < array.GetSize(); i++)
@@ -2650,6 +2637,16 @@ PStringList & PStringList::operator += (const PStringList & v)
 }
 
 
+void PStringList::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    AppendString(str);
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PSortedStringList::PSortedStringList(PINDEX count,
@@ -2671,6 +2668,12 @@ PSortedStringList::PSortedStringList(PINDEX count,
 }
 
 
+PSortedStringList::PSortedStringList(const PString & str)
+{
+  AppendString(str);
+}
+
+
 PSortedStringList::PSortedStringList(const PStringArray & array)
 {
   for (PINDEX i = 0; i < array.GetSize(); i++)
@@ -2686,6 +2689,16 @@ PSortedStringList::PSortedStringList(const PStringList & list)
 
 
 
+void PSortedStringList::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    AppendString(str);
+  }
+}
+
+
 PINDEX PSortedStringList::GetNextStringsIndex(const PString & str) const
 {
   PINDEX len = str.GetLength();
@@ -2747,6 +2760,22 @@ PStringSet::PStringSet(PINDEX count, char const * const * strarr, BOOL caseless)
 }
 
 
+PStringSet::PStringSet(const PString & str)
+{
+  Include(str);
+}
+
+
+void PStringSet::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    Include(str);
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 POrdinalToString::POrdinalToString(PINDEX count, const Initialiser * init)
@@ -2758,6 +2787,21 @@ POrdinalToString::POrdinalToString(PINDEX count, const Initialiser * init)
 }
 
 
+void POrdinalToString::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    POrdinalKey key;
+    char equal;
+    PString str;
+    strm >> key >> ws >> equal >> str;
+    if (equal != '=')
+      SetAt(key, PString::Empty());
+    else
+      SetAt(key, str.Mid(equal+1));
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PStringToOrdinal::PStringToOrdinal(PINDEX count,
@@ -2774,6 +2818,20 @@ PStringToOrdinal::PStringToOrdinal(PINDEX count,
 }
 
 
+void PStringToOrdinal::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    PINDEX equal = str.FindLast('=');
+    if (equal == P_MAX_INDEX)
+      SetAt(str, 0);
+    else
+      SetAt(str.Left(equal), str.Mid(equal+1).AsInteger());
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PStringToString::PStringToString(PINDEX count,
@@ -2797,6 +2855,20 @@ PStringToString::PStringToString(PINDEX count,
 }
 
 
+void PStringToString::ReadFrom(istream & strm)
+{
+  while (strm.good()) {
+    PString str;
+    strm >> str;
+    PINDEX equal = str.Find('=');
+    if (equal == P_MAX_INDEX)
+      SetAt(str, PString::Empty());
+    else
+      SetAt(str.Left(equal), str.Mid(equal+1));
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PRegularExpression::PRegularExpression()
