@@ -27,6 +27,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: object.cxx,v $
+ * Revision 1.65  2003/02/11 08:05:55  robertj
+ * Fixed problems in dumping memory leaks and statistics as PMemoryHeap
+ *   is flagged deleted too early, thanks Diego Trtara
+ *
+ * Revision 1.64  2003/02/06 21:03:13  dereks
+ * Patch from Klaus Kaempf to fix warning message on compiling with gcc 3.x Thanks!
+ *
  * Revision 1.63  2002/11/22 10:14:58  robertj
  * Fixed correct free of memory blocks in exceptional circumstances. Partially
  *   trashed heap or objects etc.
@@ -238,7 +245,11 @@
 #elif defined(__NUCLEUS_PLUS__)
 #include <ptlib/NucleusDebstrm.h>
 #else
+#if defined(__GNUC__) && __GNUC__ > 2
+#include <strstream>
+#else
 #include <strstream.h>
+#endif
 #include <signal.h>
 #endif
 
@@ -438,13 +449,13 @@ PMemoryHeap::PMemoryHeap()
 
 PMemoryHeap::~PMemoryHeap()
 {
-  isDestroyed = TRUE;
-
   if (leakDumpStream != NULL) {
     InternalDumpStatistics(*leakDumpStream);
     InternalDumpObjectsSince(firstRealObject, *leakDumpStream);
   }
 
+  isDestroyed = TRUE;
+
 #if defined(_WIN32)
   DeleteCriticalSection(&mutex);
   extern void PWaitOnExitConsoleWindow();
