@@ -24,6 +24,12 @@
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
  * $Log: vfakeio.cxx,v $
+ * Revision 1.17  2003/06/03 04:21:49  dereksmithies
+ * Add PTRACE statement, and tidy up format of one if statement.
+ *
+ * Revision 1.16  2003/03/17 07:46:49  robertj
+ * Migrated vflip member variable and functions into PVideoDevice class.
+ *
  * Revision 1.15  2002/09/23 07:17:24  robertj
  * Changes to allow winsock2 to be included.
  *
@@ -170,7 +176,7 @@ BOOL PFakeVideoInputDevice::SetColourFormat(const PString & newFormat)
 
 BOOL PFakeVideoInputDevice::SetFrameRate(unsigned rate)
 {
-  if ( (rate < 1) || ( rate > 50) )
+  if ((rate < 1) || (rate > 50))
     PVideoDevice::SetFrameRate(10);
   else
     PVideoDevice::SetFrameRate(rate);
@@ -218,13 +224,15 @@ void PFakeVideoInputDevice::WaitFinishPreviousFrame()
   frameTimeError -= (int)delay.GetMilliSeconds();
   previousFrameTime = now;
 
-  if (frameTimeError > 0)
+  if (frameTimeError > 0) {
+    PTRACE(6, "FakeVideo\t Sleep for " << frameTimeError << " milli seconds");
 #ifdef P_LINUX
     usleep(frameTimeError * 1000);
 #else
     PThread::Current()->Sleep(frameTimeError);
 #endif
-} 
+  } 
+}
 
 
 BOOL PFakeVideoInputDevice::GetFrameData(BYTE * buffer, PINDEX * bytesReturned)
@@ -633,23 +641,5 @@ void PFakeVideoInputDevice::GrabOriginalMovingBlocksFrame(BYTE *frame)
 }
 
 
-BOOL PFakeVideoInputDevice::GetVFlipState() 
-{
-  return doVFlip;
-} 
-
-BOOL PFakeVideoInputDevice::SetVFlipState(BOOL newVFlip) 
-{
-  doVFlip = newVFlip;
-
-  return TRUE;
-} 
-
-BOOL PFakeVideoInputDevice::ToggleVFlipState() 
-{
-  doVFlip = !doVFlip;
-
-  return TRUE;
-} 
 
 // End Of File ///////////////////////////////////////////////////////////////
