@@ -25,6 +25,13 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
  * $Log: video4linux.cxx,v $
+ * Revision 1.39  2003/03/20 23:38:06  dereks
+ * Fixes in handling of flags, and test for device opening
+ *
+ * Revision 1.38  2003/03/17 07:52:52  robertj
+ * Removed canCaptureVideo variable as this is really a virtual function to
+ *   distinguish PVideoOutputDevice from PVideoInputDevice, it is not dynamic.
+ *
  * Revision 1.37  2003/03/06 02:43:43  dereks
  * Make error messages slightly more descriptive.
  *
@@ -318,7 +325,12 @@ BOOL PVideoInputDevice::Open(const PString & devName, BOOL startImmediate)
     return FALSE;
   }
   
-  SetCanCaptureVideo(videoCapability.type & VID_TYPE_CAPTURE);
+  if ((videoCapability.type & VID_TYPE_CAPTURE) == 0) {
+    PTRACE(1,"PVideoInputDevice:: device capablilities reports cannot capture");
+    ::close (videoFd);
+    videoFd = -1;
+    return FALSE;
+  }
 
   hint_index = PARRAYSIZE(driver_hints) - 1;
   PString driver_name(videoCapability.name);  
@@ -416,8 +428,8 @@ PStringList PVideoInputDevice::GetInputDeviceNames()
 
 	if ((entry.Left(5) == "video") || (entry.Left(7) == "capture")) {
 	  PString thisDevice = "/dev/video" + entry.Right(1);
-	  int videoFd;  
-	  if ((videoFd = open(thisDevice, O_RDONLY))) {
+	  int videoFd = ::open((const char *)thisDevice, O_RDONLY);
+	  if (videoFd > 0) {
 	    struct video_capability  videoCaps;
 	    if (ioctl(videoFd, VIDIOCGCAP, &videoCaps) >= 0 &&
 		(videoCaps.type & VID_TYPE_CAPTURE) != 0)
